/*
 *
 *  (c) COPYRIGHT INRIA and W3C, 1996-2007
 *  Please first read the full copyright statement in file COPYRIGHT.
 *
 */

/*
 * This module contains functions to deal with javascript
 * thanks to SpiderMonkey (Mozilla's interpreter)
 *
 * Author: F. Wang
 */         

#ifdef _WX
#include "wx/wx.h"
#endif /* _WX */

/* Included headerfiles */
#undef THOT_EXPORT
#define THOT_EXPORT extern
#include "amaya.h"
#include "init_f.h"
#include "css.h"
#include "trans.h"
#include "undo.h"
#include "interface.h"
#include "MathML.h"
#include "parser.h"
#ifdef _SVG
#include "SVG.h"
#endif /* _SVG */
#include "document.h"

#include "fetchXMLname_f.h"
#include "SVGbuilder_f.h"
#include "html2thot_f.h"
#include "HTMLtable_f.h"
#include "HTMLactions_f.h"
#include "HTMLedit_f.h"
#include "HTMLpresentation_f.h"
#include "MathMLbuilder_f.h"
#include "styleparser_f.h"
#include "trans_f.h"
#include "UIcss_f.h"
#ifdef _WX
#include "wxdialogapi_f.h"
#endif /* _WX */
#ifdef _WINGUI
#include "wininclude.h"
#endif /* _WINGUI */
#ifdef _WINDOWS
#include <commctrl.h>
#endif /* _WINDOWS */
#include "XLinkedit_f.h"

/* headerfiles and global variables used by SpiderMonkey */
#include <jsapi.h>
static JSRuntime *grt = NULL;
static JSContext *gcx = NULL;
static JSObject *gobj = NULL;
static JSClass *gclass = NULL;
static JSBool builtins;
static Document jsdocument;

/* Define a bunch of native functions */
static JSBool js_alert(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  char *msg;
  msg = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
  TtaDisplayMessage(INFO, msg);
  rval[0] = BOOLEAN_TO_JSVAL(true);
  return JS_TRUE;
}

static JSBool js_confirm(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  char *msg;
  msg = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
  InitConfirm (jsdocument, 1, msg);
  rval[0] = BOOLEAN_TO_JSVAL(UserAnswer);
  return JS_TRUE;
}

static JSBool js_prompt(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
  char *msg;
  char *value;
  msg = JS_GetStringBytes(JS_ValueToString(cx, argv[0]));
  value = JS_GetStringBytes(JS_ValueToString(cx, argv[1]));

#ifdef _WX
  CreateTextDlgWX (BaseDialog + MathEntityForm, BaseDialog + MathEntityText, TtaGetViewFrame (jsdocument, 1), "Prompt", msg, value);
  TtaSetDialoguePosition ();
  TtaShowDialogue (BaseDialog + MathEntityForm, FALSE);
  TtaWaitShowDialogue ();
  /* TODO : get the callback and put it in rval[0] */
#endif /* _WX */

  return JS_TRUE;
}

/* Functions of the global object */
static JSFunctionSpec global_functions[] = {
  /*   name         native       nargs  */
  {"alert",        js_alert,      1},
  {"confirm",    js_confirm,      1},
  {"prompt",     js_prompt,       2},
  {0}
};

static void ExecuteScripts (Element Script);
static ThotBool IsJavascript(Element Script);
static void printError(JSContext *cx, const char *message, JSErrorReport *report);

/*----------------------------------------------------------------------
  InitJavascript
  Prepare objects and execute scripts
  -----------------------------------------------------------------------*/
void InitJavascript (Document document, View view)
{
Element child, child2;

/* Scripts can only be executed in formatted view */
if(strcmp(TtaGetViewName (document, view), "Formatted_view"))return;

jsdocument = document;

/* initialize the JS run time */
  if(!grt)grt = JS_NewRuntime(8L * 1024L * 1024L);
  if (!grt)return;

/* (re)create a context and associate it with the JS run time */
  if(gcx)JS_DestroyContext(gcx);
  gcx = JS_NewContext(grt, 8192);
  if (!gcx)return;

/* create the global object */
  gobj = JS_NewObject(gcx, gclass, NULL, NULL);

/* initialize the built-in JS objects and the global object */
  builtins = JS_InitStandardClasses(gcx, gobj);

/* initialize functions of the global object */
  JS_DefineFunctions(gcx, gobj, global_functions);

/* register error handler */
JS_SetErrorReporter(gcx, printError);

/* TODO : build classes and objects according to the document tree */

/* Search all scripts contained in the current document and execute them  */

  child = TtaGetMainRoot(document);

  while(child)
    {
    child2 = child;

    while(child2)
      {
      child = child2;

      if(IsJavascript(child))
        {
        ExecuteScripts (child);
        child2 = child;
        TtaNextSibling(&child2);
        }
      else child2 = TtaGetFirstChild(child);
      }

    child = TtaGetSuccessor(child);
    }
}

/*----------------------------------------------------------------------
  DestroyJavascript
  Free all objects used by spidermonkey
  -----------------------------------------------------------------------*/
void DestroyJavascript ()
{
if(gcx)JS_DestroyContext(gcx);
if(grt)JS_DestroyRuntime(grt);
JS_ShutDown();
}

/*----------------------------------------------------------------------
  IsJavascript
  Check whether an element is a SCRIPT_ of type javascript
  -----------------------------------------------------------------------*/
static ThotBool IsJavascript(Element element)
{
ElementType    elType;
AttributeType  attrType;
Attribute      attr;
#define BUFLEN 20
char buffer[BUFLEN];
int len;

if(!element)return FALSE;
if(TtaGetElementType(element).ElTypeNum != HTML_EL_SCRIPT_)return FALSE;

elType = TtaGetElementType (element);
attrType.AttrSSchema = elType.ElSSchema;
attrType.AttrTypeNum = HTML_ATTR_content_type;
attr = TtaGetAttribute (element, attrType);
if(!attr)return FALSE;

len = TtaGetTextAttributeLength(attr);
if(len >= BUFLEN)return FALSE;

TtaGiveTextAttributeValue(attr, buffer, &len);
if(strcmp("text/javascript", buffer))return FALSE; else return TRUE;
}

/*----------------------------------------------------------------------
  ExecuteScripts
  Execute scripts contained in an element SCRIPT_
  -----------------------------------------------------------------------*/
static void ExecuteScripts (Element Script)
{
Element child, child2, child3;
jsval rval;
char *text;
int len, i, len2;
Language	lang;

ElementType    elType;
AttributeType  attrType;
Attribute      attr;

JSScript *ScriptInFile;

/* Execute script located at the URL indicated by src */
elType = TtaGetElementType (Script);
attrType.AttrSSchema = elType.ElSSchema;
attrType.AttrTypeNum = HTML_ATTR_script_src;
attr = TtaGetAttribute (Script, attrType);
if(attr)
  {
  len = TtaGetTextAttributeLength(attr);
  text = (char *)TtaGetMemory (len + 1);
  if(text)
    {
    TtaGiveTextAttributeValue(attr, text, &len);
    ScriptInFile =  JS_CompileFile(gcx, gobj, text);
    if(ScriptInFile)JS_ExecuteScript(gcx, gobj, ScriptInFile, &rval);
    TtaFreeMemory (text);
    }
  }

/* Execute each script of (SCRIPT_)'s childs
   Normally there should only be one child but it is also possible to have something like :
    SCRIPT_
      TEXT_UNIT <-- contained only spaces and '\n'
      CDATA     <-- contained the script that have to be executed
      TEXT_UNIT <-- contained only spaces and '\n'
*/

child = TtaGetFirstChild(Script);
while(child)
  {

  switch(TtaGetElementType(child).ElTypeNum)
    {
    case HTML_EL_TEXT_UNIT:
       /* Execute the script contained in the TEXT_UNIT */
       len = TtaGetTextLength(child);
       text = (char *)TtaGetMemory (len + 1);
       if(text)
         {
         TtaGiveTextContent(child, (unsigned char *)text, &len, &lang);
         /* TODO : add name of file and line */
         JS_EvaluateScript(gcx, gobj, text, len ,NULL, 0, &rval);
         }
    break;

    case HTML_EL_CDATA:
    case HTML_EL_Comment_:

      /* Compute size of script */
      len = 0;
      child2 = TtaGetFirstChild(child);
      while(child2)
        {
        child3 = TtaGetFirstChild(child2);
        if(TtaGetElementType(child3).ElTypeNum == HTML_EL_TEXT_UNIT)len+=TtaGetTextLength(child3);
        len++;
        TtaNextSibling(&child2);
        }

      text = (char *)TtaGetMemory (len + 1);

      if(text)
        {
        /* Group all the text in one string */

        child2 = TtaGetFirstChild(child);
        i = 0;
        while(child2)
          {
          child3 = TtaGetFirstChild(child2);
          if(TtaGetElementType(child3).ElTypeNum == HTML_EL_TEXT_UNIT)
            {
            len2 = TtaGetTextLength(child3);
            TtaGiveTextContent(child3, (unsigned char *)(&(text[i])), &len2, &lang);
            i+=len2;
            }
          text[i] = '\n';i++;
          TtaNextSibling(&child2);
          }

        text[i] = '\0';
      
        /* Execute the script */
         /* TODO : add name of file and line */
        JS_EvaluateScript(gcx, gobj, text, len ,NULL, 0, &rval);
        }
    break;

    default:
    break;
    }
  if(text)TtaFreeMemory (text);

  TtaNextSibling(&child);
  }

}


/*----------------------------------------------------------------------
  printError
  Report javascript error
  -----------------------------------------------------------------------*/
static void printError(JSContext *cx, const char *message, JSErrorReport *report)
{
TtaDisplayMessage(INFO, "Javascript errors:\n\nFile: %s\nLine: %d\n\n%s",
    (report->filename ? report->filename : "NULL"),
    report->lineno,
    message
    );
}
